import { Host, h } from "@stencil/core";
export class Dropdown {
    constructor() {
        this.hasNoData = false;
        this.isPristine = true;
        this.activeItemIndex = -1;
        this.selectedOption = null;
        this.isActive = false;
        /**
         * The value of the selected dropdown item.
         */
        this.value = null;
        /**
         * Instructional text that shows before the input has a value.
         */
        this.placeholder = "";
        /**
         * If `true`, the user cannot interact with the input.
         */
        this.readonly = false;
        /**
         * If `true`, the user cannot interact with the input.
         */
        this.disabled = false;
        /**
         * If `true`, the use can search for the option.
         */
        this.typeahead = false;
        /**
         * If `true`, the height of the dropdown content is fixed.
         */
        this.fixed = true;
        /**
         * If `true`, the component uses the whole width.
         */
        this.expanded = false;
        /**
         * Defines the trigger icon on the right site.
         */
        this.triggerIcon = "caret-down";
    }
    async isActiveWatcher(newIsActive) {
        if (newIsActive) {
            if (this.typeahead) {
                this.activeItemIndex = 0;
                this.isPristine = true;
                const items = this.children;
                if (items.length > 0) {
                    const firstVisibleItemIndex = (await this.childrenWithHiddenState).indexOf(false);
                    this.activeItemIndex = firstVisibleItemIndex;
                    items.forEach((child, index) => child.activated = index === firstVisibleItemIndex);
                }
            }
            else {
                if (this.value) {
                    this.activeItemIndex = (await this.childrenWithActivatedState).indexOf(true);
                }
            }
        }
    }
    valueWatcher(newValue) {
        if (newValue) {
            this.selectedOption = this.value;
            this.inputElement.value = this.value.label;
            this.updateActivatedOptions();
        }
    }
    componentWillLoad() {
        if (this.value) {
            this.selectedOption = this.value;
        }
    }
    componentDidLoad() {
        if (this.value) {
            this.inputElement.value = this.value.label;
            this.updateActivatedOptions();
        }
    }
    disabledWatcher(newValue) {
        if (newValue === true) {
            this.isActive = false;
        }
    }
    clickOnOutside(event) {
        if (!this.element.contains(event.target) && this.isActive) {
            this.toggle();
        }
    }
    /**
     * Selects an option.
     */
    async select(option) {
        this.selectedOption = option;
        this.value = option;
        this.inputElement.value = option.label;
        this.close();
        this.balChange.emit(option);
        this.updateActivatedOptions();
    }
    /**
     * Returns the value of the dropdown.
     */
    async getSelected() {
        return this.selectedOption;
    }
    /**
     * Open & closes the dropdown.
     */
    async toggle() {
        this.isActive = !this.isActive;
    }
    /**
     * Open the dropdown menu.
     */
    async open() {
        this.isActive = true;
    }
    /**
     * Closes the dropdown menu.
     */
    async close() {
        this.isActive = false;
    }
    get children() {
        return Array.from(this.element.querySelectorAll("bal-dropdown-option"));
    }
    get childrenWithHiddenState() {
        return Promise.all(this.children.map(child => child.isHidden()));
    }
    get childrenWithActivatedState() {
        return Promise.all(this.children.map(child => child.activated));
    }
    get childrenWithValue() {
        return Promise.all(this.children.map(child => child.value));
    }
    updateActivatedOptions() {
        this.children
            .forEach(child => child.activated = child.value === this.selectedOption.value);
    }
    clicked() {
        if (!this.typeahead) {
            this.toggle();
        }
    }
    async onKeyUp(event) {
        if (!this.disabled && ["Enter", "ArrowUp", "ArrowDown"].indexOf(event.key) < 0) {
            const inputValue = event.target.value;
            this.isActive = !!inputValue;
            const children = this.children;
            if (this.typeahead && children && children.length > 0) {
                children.forEach(child => child.highlight = inputValue);
            }
            this.hasNoData = (await this.childrenWithHiddenState).every(hidden => hidden === true);
        }
    }
    handleKeyUp(event) {
        if (event.key === "Escape" || event.key === "Esc") {
            event.preventDefault();
            this.isActive = false;
        }
    }
    async handleKeyDown(event) {
        if (event.key === "ArrowUp" || event.key === "ArrowDown" || event.key === "Down" || event.key === "Up") {
            event.preventDefault();
            await this.focusNextItem(event.key === "ArrowDown" || event.key === "Down");
        }
        if (event.key === "Enter") {
            event.preventDefault();
            this.select({
                label: this.children[this.activeItemIndex].label,
                value: this.children[this.activeItemIndex].value,
            });
        }
    }
    async getNextItem() {
        let hasSetIndex = false;
        let nextIndex;
        if (this.activeItemIndex >= 0) {
            nextIndex = this.activeItemIndex;
        }
        (await this.childrenWithHiddenState).forEach((isHidden, index) => {
            if (index > this.activeItemIndex && !isHidden && !hasSetIndex) {
                nextIndex = index;
                hasSetIndex = true;
            }
        });
        return nextIndex;
    }
    async getPreviousItem() {
        let previousIndex = this.activeItemIndex;
        (await this.childrenWithHiddenState).forEach((isHidden, index) => {
            if (index < this.activeItemIndex && !isHidden) {
                previousIndex = index;
            }
        });
        return previousIndex;
    }
    async focusNextItem(isArrowDown) {
        const children = this.children;
        if (children && children.length > 0 && !this.disabled) {
            if (isArrowDown) {
                this.activeItemIndex = await this.getNextItem();
            }
            else {
                this.activeItemIndex = await this.getPreviousItem();
            }
            const scrollPuffer = this.element.offsetTop + 45;
            const activeChild = children[this.activeItemIndex] || children[0];
            const visMin = this.dropdownContentElement.scrollTop + scrollPuffer;
            const visMax = this.dropdownContentElement.scrollTop + scrollPuffer
                + this.dropdownContentElement.clientHeight - activeChild.clientHeight;
            if (activeChild.offsetTop < visMin) {
                this.dropdownContentElement.scrollTop = activeChild.offsetTop - scrollPuffer;
            }
            else if (activeChild.offsetTop >= visMax) {
                this.dropdownContentElement.scrollTop = (activeChild.offsetTop - scrollPuffer -
                    this.dropdownContentElement.clientHeight +
                    activeChild.clientHeight);
            }
            children
                .forEach((child, index) => child.activated = index === this.activeItemIndex);
        }
    }
    render() {
        return (h(Host, null,
            h("div", { class: [
                    "dropdown",
                    this.expanded ? "is-fullwidth" : "",
                    this.isActive ? "is-active" : "",
                    this.fixed ? "is-fixed" : "",
                    this.typeahead ? "is-typeahead" : "",
                ].join(" ") },
                h("div", { class: "dropdown-trigger" },
                    h("div", { class: [
                            "control",
                            this.triggerIcon && !this.typeahead ? "has-icons-right" : "",
                        ].join(" ") },
                        h("input", { class: [
                                "input",
                                this.isActive ? "is-focused" : "",
                            ].join(" "), autocomplete: "off", disabled: this.disabled, readOnly: this.readonly || !this.typeahead, placeholder: this.placeholder, onKeyUp: this.onKeyUp.bind(this), onClick: this.clicked.bind(this), onBlur: this.balBlur.emit.bind(this), onFocus: this.balFocus.emit.bind(this), ref: el => this.inputElement = el }),
                        h("bal-icon", { size: "small", name: this.triggerIcon, isRight: true, style: { display: this.triggerIcon && !this.typeahead ? "flex" : "none" } }))),
                h("div", { class: "dropdown-menu", role: "menu" },
                    h("div", { class: "dropdown-content", ref: el => this.dropdownContentElement = el },
                        h("slot", null),
                        h("span", { class: "no-data", style: !this.hasNoData && { display: "none" } },
                            h("slot", { name: "no-data-content" }, "No Data")))))));
    }
    static get is() { return "bal-dropdown"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() { return {
        "$": ["dropdown.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["dropdown.css"]
    }; }
    static get properties() { return {
        "value": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "Option",
                "resolved": "Option",
                "references": {
                    "Option": {
                        "location": "import",
                        "path": "../dropdown-option/dropdown-option.types"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The value of the selected dropdown item."
            },
            "defaultValue": "null"
        },
        "placeholder": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Instructional text that shows before the input has a value."
            },
            "attribute": "placeholder",
            "reflect": false,
            "defaultValue": "\"\""
        },
        "readonly": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If `true`, the user cannot interact with the input."
            },
            "attribute": "readonly",
            "reflect": false,
            "defaultValue": "false"
        },
        "disabled": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If `true`, the user cannot interact with the input."
            },
            "attribute": "disabled",
            "reflect": false,
            "defaultValue": "false"
        },
        "typeahead": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If `true`, the use can search for the option."
            },
            "attribute": "typeahead",
            "reflect": false,
            "defaultValue": "false"
        },
        "fixed": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If `true`, the height of the dropdown content is fixed."
            },
            "attribute": "fixed",
            "reflect": false,
            "defaultValue": "true"
        },
        "expanded": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If `true`, the component uses the whole width."
            },
            "attribute": "expanded",
            "reflect": false,
            "defaultValue": "false"
        },
        "triggerIcon": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Defines the trigger icon on the right site."
            },
            "attribute": "trigger-icon",
            "reflect": false,
            "defaultValue": "\"caret-down\""
        }
    }; }
    static get states() { return {
        "selectedOption": {},
        "isActive": {}
    }; }
    static get events() { return [{
            "method": "balChange",
            "name": "balChange",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emitted when the checked property has changed."
            },
            "complexType": {
                "original": "Option",
                "resolved": "Option",
                "references": {
                    "Option": {
                        "location": "import",
                        "path": "../dropdown-option/dropdown-option.types"
                    }
                }
            }
        }, {
            "method": "balBlur",
            "name": "balBlur",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emitted when the toggle loses focus."
            },
            "complexType": {
                "original": "void",
                "resolved": "void",
                "references": {}
            }
        }, {
            "method": "balFocus",
            "name": "balFocus",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emitted when the toggle has focus.."
            },
            "complexType": {
                "original": "void",
                "resolved": "void",
                "references": {}
            }
        }]; }
    static get methods() { return {
        "select": {
            "complexType": {
                "signature": "(option: Option) => Promise<void>",
                "parameters": [{
                        "tags": [],
                        "text": ""
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    },
                    "Option": {
                        "location": "import",
                        "path": "../dropdown-option/dropdown-option.types"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Selects an option.",
                "tags": []
            }
        },
        "getSelected": {
            "complexType": {
                "signature": "() => Promise<Option>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    },
                    "Option": {
                        "location": "import",
                        "path": "../dropdown-option/dropdown-option.types"
                    }
                },
                "return": "Promise<Option>"
            },
            "docs": {
                "text": "Returns the value of the dropdown.",
                "tags": []
            }
        },
        "toggle": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Open & closes the dropdown.",
                "tags": []
            }
        },
        "open": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Open the dropdown menu.",
                "tags": []
            }
        },
        "close": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Closes the dropdown menu.",
                "tags": []
            }
        }
    }; }
    static get elementRef() { return "element"; }
    static get watchers() { return [{
            "propName": "isActive",
            "methodName": "isActiveWatcher"
        }, {
            "propName": "value",
            "methodName": "valueWatcher"
        }, {
            "propName": "disabled",
            "methodName": "disabledWatcher"
        }]; }
    static get listeners() { return [{
            "name": "click",
            "method": "clickOnOutside",
            "target": "document",
            "capture": false,
            "passive": false
        }, {
            "name": "keyup",
            "method": "handleKeyUp",
            "target": undefined,
            "capture": false,
            "passive": false
        }, {
            "name": "keydown",
            "method": "handleKeyDown",
            "target": undefined,
            "capture": false,
            "passive": false
        }]; }
}
